<?php
/**
* Created on Mar 7, 2013
* @package   prj_mojeid_j25
* @author    Filip Bartmann, Martin Kušnir
* @copyright Copyright (C) WebSite21 | www.website21.cz | www.spaceshop.cz | www.joomladev.eu
* @license   http://www.gnu.org/licenses/gpl-3.0.html | http://www.gnugpl.cz/v3/
*/

defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');
jimport('joomla.filesystem.folder');

class plgAuthenticationMojeID extends JPlugin {
	function __construct($config,$params) {
		parent::__construct($config,$params);
		$this->loadLanguage();
	}
	
	function onUserAuthenticate(&$credentials,$options,&$response) {
		require_once JPATH_LIBRARIES.'/lib_mojeid_openid_j25/common.php';		
		
		$consumer=getConsumer();
		$result=$consumer->complete(getReturnTo());
		
		switch($result->status) {
			case Auth_OpenID_CANCEL:
				$response->status=JAuthentication::STATUS_CANCEL;
				$response->error_message=JText::_('PLG_AUTHENTICATE_MOJEID_AUTHENTICATION_CANCELLED');
				break;
			case Auth_OpenID_FAILURE:
				$response->status=JAuthentication::STATUS_FAILURE;
				$response->error_message=JText::sprintf('PLG_AUTHENTICATE_MOJEID_AUTHENTICATION_FAILED',$result->message);
				break;
			case Auth_OpenID_SUCCESS:			
				$response->status=JAuthentication::STATUS_SUCCESS;
				$response->error_message="";
				
				if(!$this->_saveUser($response,$result)) {
					echo $response->error_message;exit;
					return false;
				}
				
				break;
		}
	}
	
	private function _saveUser(&$response,$result) {
		$db=JFactory::getDBO();
				
		$query=$db->getQuery(true);
		$query->select('um.user_id, u.username');
		$query->from('#__user_mojeid AS um');
		$query->leftJoin('#__users AS u ON um.user_id=u.id');
		$query->where('mojeid_ident='.$db->quote(htmlentities($result->getDisplayIdentifier())));
		$db->setQuery($query);
		$mojeid_o=$db->loadObject();		
		
		$response->username=htmlentities($result->getDisplayIdentifier());
		$response->username=str_replace(array("http://","https://","/"),"",$response->username);
		$response->username=str_replace(array(".fred.nic.cz"),"",$response->username);
		$response->username=str_replace('.mojeid','',$response->username);
		
		$data_resp=Auth_OpenID_SRegResponse::fromSuccessResponse($result);
		$data=$data_resp->contents();
		if(isset($data["email"])) {
			$response->email=$data["email"];
		} else {
			$response->email=str_replace(array("http://","https://"),"",$response->username);
			$response->email=str_replace('/','-',$response->email);
			$response->email.='@mojeid.';
		}
		$response->fullname=isset($data["fullname"])?$data["fullname"]:$response->username;
		$response->language=isset($data["language"])?$data["language"]:null;
		$response->timezone=isset($data["timezone"])?$data["timezone"]:null;
		
		$ax=Auth_OpenID_AX_FetchResponse::fromSuccessResponse($result);
		
		if($this->params->get('adult_protect',0)==1) {
			if(!isset($ax->data["http://specs.nic.cz/attr/contact/valid"][0]) || strtolower($ax->data["http://specs.nic.cz/attr/contact/valid"][0])=="false") {
				$response->status=JAuthentication::STATUS_FAILURE;
				$response->error_message=JText::_('PLG_AUTHENTICATE_MOJEID_AUTHENTICATION_FAILED_PROFILE_NOT_VALIDATED');
				return false;					
			} else if(strtolower($ax->data["http://specs.nic.cz/attr/contact/adult"][0])=="false") {
				$response->status=JAuthentication::STATUS_FAILURE;
				$response->error_message=JText::_('PLG_AUTHENTICATE_MOJEID_AUTHENTICATION_FAILED_AGE');
				return false;
			}
		}
		
		$user=JUser::getInstance();
		if(JFactory::getUser()->id>0) {
			$user->load(JFactory::getUser()->id);
			$user->set('username',$response->username);
		} else {
			if(($user_id=JUserHelper::getUserId($response->username))) {
				$user->load($user_id);	
			}
			$user->set('username',$response->username);
		}
		$user->set('email',$response->email);
		
		$old_local_exists=false;
		if(JFactory::getUser()->id>0 && $mojeid_o->user_id!=$user->id) {
			$old_local_exists=true;
		}
		
		$user->set('name',$response->fullname);
		if($user->get('id')==0) {
			$user->set('groups',array(JComponentHelper::getParams('com_users')->get('new_usertype',2)));
		}

		if(!$old_local_exists) {
			if(!$user->save()) {
				$response->status=JAuthentication::STATUS_FAILURE;
				$response->error_message=JText::sprintf('PLG_AUTHENTICATE_MOJEID_AUTHENTICATION_FAILED',$user->getError());
				return false;
			}
		}
		
		if(!$old_local_exists) {
			$query=$db->getQuery(true);
			$query->insert('#__user_mojeid');
			$query->set('user_id='.$user->id);
			$query->set('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$query.=" ON DUPLICATE KEY UPDATE mojeid_ident=".$db->quote($result->getDisplayIdentifier());
			$db->setQuery($query);
			$db->execute();
		} else {
			$query=$db->getQuery(true);
			$query->update('#__user_mojeid');
			$query->set('user_id='.$user->id);
			$query->where('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$db->setQuery($query);
			$db->execute();
		}
		
		if(JPluginHelper::isEnabled('user','profile')) {
			$ax_data=$ax->data;
			$data=array();
			if(!empty($ax_data['http://axschema.org/contact/city/home'][0])) {
				$data['city']=$ax_data['http://axschema.org/contact/city/home'][0];
			}
			if(!empty($ax_data['http://axschema.org/contact/postalAddress/home'][0])) {
				$data['address1']=$ax_data['http://axschema.org/contact/postalAddress/home'][0];
			}
			if(!empty($ax_data['http://axschema.org/contact/postalCode/home'][0])) {
				$data['postal_code']=$ax_data['http://axschema.org/contact/postalCode/home'][0];
			}
			if(!empty($ax_data['http://axschema.org/contact/phone/default'][0])) {
				$data['phone']=$ax_data['http://axschema.org/contact/phone/default'][0];
			} else if(!empty($ax_data['http://axschema.org/contact/phone/home'][0])) {
				$data['phone']=$ax_data['http://axschema.org/contact/phone/home'][0];
			}
			if(!empty($ax_data['http://axschema.org/contact/web/default'][0])) {
				$data['website']=$ax_data['http://axschema.org/contact/web/default'][0];
			}
			if(!empty($ax_data['http://specs.nic.cz/attr/contact/ident/dob'][0])) {
				$data['dob']=$ax_data['http://specs.nic.cz/attr/contact/ident/dob'][0];
			}
			
			
			foreach($data as $key=>$val) {
				$query=$db->getQuery(true);
				$query->select('COUNT(*)');
				$query->from('#__user_profiles');
				$query->where('user_id='.$user->get('id'));
				$query->where('profile_key='.$db->quote('profile.'.$key));
				$db->setQuery($query);
				$new=!(bool)$db->loadResult();
			
				$query=$db->getQuery(true);
				if($new) {
					$query->insert('#__user_profiles');
					$query->set('user_id='.$user->get('id'));
				} else {
					$query->update('#__user_profiles');
				}
				$query->set('profile_key='.$db->quote('profile.'.$key));
				$query->set('profile_value='.$db->quote(json_encode($val)));			
				
				if(!$new) {
					$query->where('user_id='.$user->get('id'));	
					$query->where('profile_key='.$db->quote('profile.'.$key));
				}
				$db->setQuery($query);
				if(!$db->query()) {
					$this->setError($db->stderr(true));
					return false;
				}
			}
			
			if($old_local_exists) {
				$query=$db->getQuery(true);
				$query->select('REPLACE(profile_key,"profile.","") AS pkey, profile_value');
				$query->from('#__user_profiles');
				$query->where('user_id='.$mojeid_o->user_id);
				$query->where('profile_key LIKE ('.$db->quote('profile.%').')');
				$db->setQuery($query);
				$profile=$db->loadObjectList('pkey');
				
				foreach($profile as $key=>$val) {
					$query=$db->getQuery(true);
					$query->select('profile_value');
					$query->from('#__user_profiles');
					$query->where('user_id='.$user->get('id'));
					$query->where('profile_key='.$db->quote('profile.'.$key));
					$db->setQuery($query);
					$profile_value=$db->loadResult();
					if($val->profile_value!='""') {
						$query=$db->getQuery(true);
						$query->select('COUNT(*)');
						$query->from('#__user_profiles');
						$query->where('user_id='.$user->get('id'));
						$query->where('profile_key='.$db->quote('profile.'.$key));
						$db->setQuery($query);
						$cnt=$db->loadResult();
						
						$query=$db->getQuery(true);
						if($cnt) {
							$query->update('#__user_profiles');
							$query->where('user_id='.$user->get('id'));
							$query->where('profile_key='.$db->quote('profile.'.$key));	
						} else {
							$query->insert('#__user_profiles');
							$query->set('user_id='.$user->get('id'));
							$query->set('profile_key='.$db->quote('profile.'.$key));
						}
						$query->set('profile_value='.$db->quote($val->profile_value));
						$db->setQuery($query);
						if(!$db->query()) {
							$this->setError($this->_db->stderr(true));
							return false;
						}
					}
				}
				
				$query=$db->getQuery(true);
				$query->delete('#__user_profiles');
				$query->where('user_id='.$mojeid_o->user_id);
				$query->where('profile_key LIKE ('.$db->quote('profile.%').')');
				$db->setQuery($query);
				if(!$db->query()) {
					$this->setError($db->stderr(true));
					return false;
				}
			}
		}
		
		if(JFolder::exists(JPATH_ADMINISTRATOR.'/components/com_virtuemart') && JComponentHelper::isEnabled('com_virtuemart')) {
			if(!$this->_saveVirtueMart($user,$response,$result,$old_local_exists,$mojeid_o)) { 
				return false;
			}
		}
		if(JFolder::exists(JPATH_ADMINISTRATOR.'/components/com_hikashop') && JComponentHelper::isEnabled('com_hikashop')) {
			if(!$this->_saveHikaShop($user,$response,$result,$old_local_exists,$mojeid_o)) { 
				return false;
			}
		}
		if(JFolder::exists(JPATH_ADMINISTRATOR.'/components/com_spaceshop') && JComponentHelper::isEnabled('com_spaceshop')) {
			if(!$this->_saveSPACEShop($user,$response,$result,$old_local_exists,$mojeid_o)) { 
				return false;
			}
		}
		
		JPluginHelper::importPlugin('mojeid');
		JDispatcher::getInstance()->trigger('onSave',array($user,&$response,$result,$old_local_exists,$old_local_exists?$mojeid_o->user_id:0));
		
		
		if($old_local_exists) {
			$old_local_user=JUser::getInstance();		
			$old_local_user->load($mojeid_o->user_id);
			$old_local_user->email="none@none.none";
			$old_local_user->delete();
			
			if(!$user->save()) {
				$response->status=JAuthentication::STATUS_FAILURE;
				$response->error_message=JText::sprintf('PLG_AUTHENTICATE_MOJEID_AUTHENTICATION_FAILED',$user->getError());			
				return false;
			}
		}
		
		return true;
	}
	
	private function _saveVirtueMart($user,&$response,$result,$old_local_exists,$mojeid_o) {
		$db=JFactory::getDBO();
		$query=$db->getQuery(true);
		$query->select('COUNT(*)');
		$query->from('#__virtuemart_userinfos');
		$query->where('virtuemart_user_id='.$user->get('id'));
		$query->where('address_type='.$db->quote('BT'));
		$db->setQuery($query);
		$bt_new=!(bool)$db->loadResult();
		
		if(!$old_local_exists) {
			$query=$db->getQuery(true);
			$query->select('COUNT(*)');
			$query->from('#__virtuemart_userinfos');
			$query->where('virtuemart_user_id='.$user->get('id'));
			$query->where('address_type='.$db->quote('ST'));
			$query->where('address_type_name='.$db->quote('MojeID'));
			
			$db->setQuery($query);
			$st_new=!(bool)$db->loadResult();
		} else {
			$st_new=true;
		}
		
		$bt_query=$db->getQuery(true);
		if($bt_new) {
			$bt_query->insert('#__virtuemart_userinfos');
			$bt_query->set('virtuemart_user_id='.$user->get('id'));
			$bt_query->set('address_type='.$db->quote('BT'));
		} else {
			$bt_query->update('#__virtuemart_userinfos');
			$bt_query->where('virtuemart_user_id='.$user->get('id'));	
			$bt_query->where('address_type='.$db->quote('BT'));
		}
		
		$st_query=$db->getQuery(true);
		if($st_new) {
			$st_query->insert('#__virtuemart_userinfos');
			$st_query->set('virtuemart_user_id='.$user->get('id'));
			$st_query->set('address_type='.$db->quote('ST'));
		} else {
			$st_query->update('#__virtuemart_userinfos');
			$st_query->where('virtuemart_user_id='.$user->get('id'));
			$st_query->where('address_type='.$db->quote('ST'));	
			$st_query->where('address_type_name='.$db->quote('MojeID'));
		}
		
		$name=explode(" ",$response->fullname);
		$bt_query->set('first_name='.$db->quote($name[0]));
		$bt_query->set('last_name='.$db->quote($name[1]));
		
		$st_query->set('first_name='.$db->quote($name[0]));
		$st_query->set('last_name='.$db->quote($name[1]));
		$st_query->set('address_type_name='.$db->quote('MojeID'));
		
		$st_used=false;
		
		$ax=Auth_OpenID_AX_FetchResponse::fromSuccessResponse($result);
		foreach($ax->data as $uri=>$vals) {
			$name=str_replace("http://specs.nic.cz/attr/","",$uri);
			switch($name) {
				case 'addr/bill/street':
					if(isset($vals[0])) {
						$bt_query->set('address_1='.$db->quote($vals[0]));
					}
					break;
				case 'addr/bill/city':
					if(isset($vals[0])) {
						$bt_query->set('city='.$db->quote($vals[0]));
					}
					break;
				case 'addr/bill/pc':
					if(isset($vals[0])) {
						$bt_query->set('zip='.$db->quote($vals[0]));	
					}
					break;
				case 'addr/bill/cc':
					if(isset($vals[0])) {
						$country_query=$db->getQuery(true);
						$country_query->select('virtuemart_country_id');
						$country_query->from('#__virtuemart_countries');
						$country_query->where('country_2_code='.$db->quote($vals[0]));
						$db->setQuery($country_query);
						$bt_query->set('virtuemart_country_id='.$db->loadResult());
					}
					break;
				case 'http://axschema.org/contact/phone/default':
					if(!empty($vals[0])) {
						$bt_query->set('phone_1='.$db->quote($vals[0]));
					}
					break;
				case 'http://axschema.org/contact/phone/home':
					if(!empty($vals[0])) {
						$bt_query->set('phone_1='.$db->quote($vals[0]));
					}
					break;
												
				case 'addr/ship/street':
					if(isset($vals[0])) {
						$st_query->set('address_1='.$db->quote($vals[0]));
						$st_used=true;
					}
					break;
				case 'addr/ship/city':
					if(isset($vals[0])) {
						$st_query->set('city='.$db->quote($vals[0]));
						$st_used=true;
					}
					break;
				case 'addr/ship/pc':
					if(isset($vals[0])) {
						$st_query->set('zip='.$db->quote($vals[0]));
						$st_used=true;
					}
					break;		
				case 'addr/ship/cc':
					if(isset($vals[0])) {
						$country_query=$db->getQuery(true);
						$country_query->select('virtuemart_country_id');
						$country_query->from('#__virtuemart_countries');
						$country_query->where('country_2_code='.$db->quote($vals[0]));
						$db->setQuery($country_query);
						$st_query->set('virtuemart_country_id='.$db->loadResult());
						$st_used=true;
					}
				
			}
		}
		$db->setQuery($bt_query);
		if(!$db->query()) {
			$this->setError($db->stderr(true));
			return false;
		}
		
		if($st_used) {
			$db->setQuery($st_query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
		}
		
		if($old_local_exists) {
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__virtuemart_userinfos');
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$query->where('address_type='.$db->quote('BT'));
			$db->setQuery($query);
			$mojeid_bt_address=$db->loadObjectList();
			
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__virtuemart_userinfos');
			$query->where('virtuemart_user_id='.$user->get('id'));
			$query->where('address_type='.$db->quote('BT'));
			$db->setQuery($query);
			$local_bt_address=$db->loadObjectList();
			
			$query=$db->getQuery(true);
			if(is_null($local_bt_address)) {
				$query->insert('#__virtuemart_userinfos');
				$query->set('virtuemart_user_id='.$user->get('id'));
				$query->set('address_type='.$db->quote('BT'));
			} else {
				$query->update('#__virtuemart_userinfos');
				$query->where('virtuemart_user_id='.$user->get('id'));
				$query->where('address_type='.$db->quote('BT'));
				
				// Ensure when no update found, that query dont fail
				$query->set('virtuemart_user_id='.$user->get('id'));
			}
			
			foreach($mojeid_bt_address as $key=>$val) {
				if(!empty($mojeid_bt_address->$key)) {
					$query->set($key.'='.$db->quote($val));
				}
			}
			
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__virtuemart_userinfos');
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$query->where('address_type='.$db->quote('ST'));
			$query->where('address_type_name='.$db->quote('MojeID'));
			$db->setQuery($query);
			$mojeid_st_address=$db->loadObjectList();
			
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__virtuemart_userinfos');
			$query->where('virtuemart_user_id='.$user->get('id'));
			$query->where('address_type='.$db->quote('ST'));
			$query->where('address_type_name='.$db->quote('MojeID'));
			$db->setQuery($query);
			$local_st_address=$db->loadObjectList();
			
			$query=$db->getQuery(true);
			if(is_null($local_st_address)) {
				$query->insert('#__virtuemart_userinfos');
				$query->set('virtuemart_user_id='.$user->get('id'));
				$query->set('address_type='.$db->quote('ST'));
			} else {
				$query->update('#__virtuemart_userinfos');
				$query->where('virtuemart_user_id='.$user->get('id'));
				$query->where('address_type='.$db->quote('ST'));
				// Ensure when no update found, that query dont fail
				$query->set('virtuemart_user_id='.$user->get('id'));
			}
			
			foreach($mojeid_st_address as $key=>$val) {
				if(!empty($mojeid_st_address->$key)) {
					$query->set($key.'='.$db->quote($val));
				}
			}
			
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__virtuemart_userinfos');
			$query->set('virtuemart_user_id='.$user->get('id'));
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$query->where('address_type_name!='.$db->quote('MojeID'));
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->delete('#__virtuemart_userinfos');
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__virtuemart_carts');
			$query->set('virtuemart_user_id='.$user->get('id'));
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__virtuemart_orders');
			$query->set('virtuemart_user_id='.$user->get('id'));
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->select('virtuemart_shoppergroup_id');
			$query->from('#__virtuemart_vmuser_shoppergroups');
			$query->where('virtuemart_user_id='.$user->get('id'));
			$db->setQuery($query);
			$virtuemart_shoppergroup_ids=$db->loadColumn();
			
			$query=$db->getQuery(true);
			$query->delete('#__virtuemart_vmuser_shoppergroups');
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			if(!empty($virtuemart_shoppergroup_ids)) {
				$query->where('virtuemart_shoppergroup_id IN('.implode(",",$virtuemart_shoppergroup_ids).')');
			}
			$db->setQUery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
						
			$query=$db->getQuery(true);
			$query->update('#__virtuemart_vmuser_shoppergroups');
			$query->set('virtuemart_user_id='.$user->get('id'));
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->select('virtuemart_vendor_id, user_is_vendor');
			$query->from('#__virtuemart_vmusers');
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			$vmuser=$db->loadObject();
			if($vmuser->user_is_vendor==1) {
				$query=$db->getQuery(true);
				$query->update('#__virtuemart_vmusers');
				$query->set('virtuemart_vendor_id='.$vmuser->virtuemart_vendor_id);
				$query->set('user_is_vendor=1');
				$query->where('virtuemart_user_id='.$user->get('id'));
				$db->setQuery($query);
				if(!$db->query()) {
					$this->setError($db->stderr(true));
					return false;
				}
			}
			
			$query=$db->getQuery(true);
			$query->select('virtuemart_product_id');
			$query->from('#__virtuemart_waitingusers');
			$query->where('virtuemart_user_id='.$user->get('id'));
			$db->setQuery($query);
			$virtuemart_product_ids=$db->loadColumn();
			
			$query=$db->getQuery(true);
			$query->update('#__virtuemart_waitingusers');
			$query->set('virtuemart_user_id='.$user->get('id'));
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			if(!empty($virtuemart_product_ids)) {
				$query->where('virtuemart_product_id NOT IN('.implode(",",$virtuemart_product_ids).')');	
			}
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->delete('#__virtuemart_waitingusers');
			$query->where('virtuemart_user_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
		}
		return true;
	}
	
	private function _saveHikaShop($user,&$response,$result,$old_local_exists,$mojeid_o) {
		$db=JFactory::getDBO();
		
		$query=$db->getQuery(true);
		$query->select('user_id');
		$query->from('#__hikashop_user');
		$query->where('user_cms_id='.$user->get('id'));
		$db->setQuery($query);
		$hika_user_id=$db->loadResult();	
		
		if(is_null($hika_user_id)) {
			$query=$db->getQuery(true);
			$query->insert('#__hikashop_user');
			$query->set('user_cms_id='.$user->get('id'));
			$query->set('user_email='.$db->quote($user->get('email')));
			$query->set('user_created='.JFactory::getDate()->toUnix());
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;	
			}
		} else {
			// Because of uniqueness of e-mail field - we will have same e-mail at old moejeId user account and current
			if(!$old_local_exists) {
				$query=$db->getQuery(true);
				$query->update('#__hikashop_user');
				$query->set('user_email='.$db->quote($user->get('email')));
				$query->where('user_cms_id='.$user->get('id'));
				$db->setQuery($query);
				if(!$db->query()) {
					$this->setError($db->stderr(true));
					return false;	
				}
			}
		}
		
		$address_bt_id=null;
		if(!is_null($hika_user_id)) {
			$query=$db->getQuery(true);
			$query->select('address_id');
			$query->from('#__hika_address_mojeid');
			$query->where('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$query->where('address_type='.$db->quote('BT'));
			$db->setQuery($query);
			$address_bt_id=$db->loadResult();
		}
		
		$address_st_id=null;
		if(!is_null($hika_user_id)) {
			$query=$db->getQuery(true);
			$query->select('address_id');
			$query->from('#__hika_address_mojeid');
			$query->where('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$query->where('address_type='.$db->quote('ST'));
			$db->setQuery($query);
			$address_st_id=$db->loadResult();
		}
		
		$address_bt_query=$db->getQuery(true);
		if(is_null($address_bt_id)) {
			$address_bt_query->insert('#__hikashop_address');
			$address_bt_query->set('address_user_id='.$hika_user_id);
			$address_bt_query->set('address_default=1');
			$address_bt_query->set('address_published=1');
		} else {
			$address_bt_query->update('#__hikashop_address');
			$address_bt_query->where('address_id='.$address_bt_id);
		}	
		
		$address_st_query=$db->getQuery(true);
		if(is_null($address_st_id)) {
			$address_st_query->insert('#__hikashop_address');
			$address_st_query->set('address_user_id='.$hika_user_id);
			$address_st_query->set('address_default=0');
			$address_st_query->set('address_published=1');
		} else {
			$address_st_query->update('#__hikashop_address');
			$address_st_query->where('address_id='.$address_st_id);
		}		
		
		$name=explode(" ",$response->fullname);
		$address_bt_query->set('address_firstname='.$db->quote($name[0]));
		$address_bt_query->set('address_lastname='.$db->quote($name[1]));
		
		$address_st_query->set('address_firstname='.$db->quote($name[0]));
		$address_st_query->set('address_lastname='.$db->quote($name[1]));
		
		$ax=Auth_OpenID_AX_FetchResponse::fromSuccessResponse($result);
		foreach($ax->data as $uri=>$vals) {
			$name=str_replace("http://specs.nic.cz/attr/","",$uri);
			switch($name) {
				case 'addr/bill/street':
					if(isset($vals[0])) {
						$address_bt_query->set('address_street='.$db->quote($vals[0]));
					}
					break;
				case 'addr/bill/city':
					if(isset($vals[0])) {
						$address_bt_query->set('address_city='.$db->quote($vals[0]));
					}
					break;
				case 'addr/bill/pc':
					if(isset($vals[0])) {
						$address_bt_query->set('address_post_code='.$db->quote($vals[0]));	
					}
					break;
				case 'addr/bill/cc':
					if(isset($vals[0])) {
						$country_query=$db->getQuery(true);
						$country_query->select('CONCAT("country_",zone_name_english,"_",zone_id)');
						$country_query->from('#__hikashop_zone');
						$country_query->where('zone_code_2='.$db->quote($vals[0]));
						$country_query->where('zone_type='.$db->quote('country'));
						$db->setQuery($country_query);
						$address_bt_query->set('address_country='.$db->quote(str_replace(' ','_',$db->loadResult())));
					}
					break;
				case 'http://axschema.org/contact/phone/default':
					if(!empty($vals[0])) {
						$address_bt_query->set('address_telephone='.$db->quote($vals[0]));
					}
					break;
				case 'http://axschema.org/contact/phone/home':
					if(!empty($vals[0])) {
						$address_bt_query->set('address_telephone2='.$db->quote($vals[0]));
					}
					break;
												
				case 'addr/ship/street':
					if(isset($vals[0])) {
						$address_st_query->set('address_street='.$db->quote($vals[0]));
					}
					break;
				case 'addr/ship/city':
					if(isset($vals[0])) {
						$address_st_query->set('address_city='.$db->quote($vals[0]));
					}
					break;
				case 'addr/ship/pc':
					if(isset($vals[0])) {
						$address_st_query->set('address_post_code='.$db->quote($vals[0]));
					}
					break;		
				case 'addr/ship/cc':
					if(isset($vals[0])) {
						$country_query=$db->getQuery(true);
						$country_query->select('CONCAT("country_",zone_name_english,"_",zone_id)');
						$country_query->from('#__hikashop_zone');
						$country_query->where('zone_code_2='.$db->quote($vals[0]));
						$country_query->where('zone_type='.$db->quote('country'));
						$db->setQuery($country_query);
						$address_st_query->set('address_country='.$db->quote(str_replace(' ','_',$db->loadResult())));
					}
					break;
				
			}
		}
		if(!is_null($address_bt_id)) {
			$address_bt_query->where('address_id='.$address_bt_id);	
		}
		$db->setQuery($address_bt_query);
		if(!$db->query()) {
			$this->setError($db->stderr(true));
			return false;
		}
		if(is_null($address_bt_id)) {
			$address_id=$db->insertid();
			
			$query=$db->getQuery(true);
			$query->insert('#__hika_address_mojeid');
			$query->set('address_id='.$address_id);
			$query->set('address_type='.$db->quote('BT'));
			$query->set('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
		}
		
		if(!is_null($address_st_id)) {
			$address_bt_query->where('address_id='.$address_st_id);	
		}
		$db->setQuery($address_st_query);
		if(!$db->query()) {
			$this->setError($db->stderr(true));
			return false;
		}
		
		if(is_null($address_st_id)) {
			$address_id=$db->insertid();
			
			$query=$db->getQuery(true);
			$query->insert('#__hika_address_mojeid');
			$query->set('address_id='.$address_id);
			$query->set('address_type='.$db->quote('ST'));
			$query->set('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($this->_db->stderr(true));
				return false;
			}
		}
		if($old_local_exists) {
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__hikashop_user');
			$query->where('user_cms_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			$mojeid_hikashop_user=$db->loadObject();
			$mojeid_hikashop_user_id=$mojeid_hikashop_user->user_id;
			
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__hikashop_user');
			$query->where('user_cms_id='.$user->get('id'));
			$db->setQuery($query);
			$local_hikashop_user=$db->loadObject();
			$local_hikashiop_user_id=$local_hikashop_user->user_id;
			
			$query=$db->getQuery(true);
			
			if(is_null($local_hikashop_user)) {
				$query->insert('#__hikashop_user');
				$query->set('user_cms_id='.$user->get('id'));
			} else {
				$query->update('#__hikashop_user');
				$query->where('user_cms_id='.$user->get('id'));
				$query->set('user_cms_id='.$user->get('id'));
			}
			
			foreach(get_object_vars($mojeid_hikashop_user) as $key=>$val) {
				if(empty($local_hikashop_user->$key) && !empty($val)) {
					$query->set($key.'='.$db->quote($val));		
				}
			}
			
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->select('address_id');
			$query->from('#__hika_address_mojeid');
			$query->where('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$query->where('address_type='.$db->quote('BT'));
			$db->setQuery($query);
			$moje_id_bt_address_id=$db->loadResult();
			
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__hikashop_address');
			$query->where('address_user_id='.$mojeid_hikashop_user_id);
			$query->where('address_default=1');
			$query->where('address_id='.$moje_id_bt_address_id);
			$db->setQuery($query);
			$mojeid_hikashop_address=$db->loadObject();
			
			$query=$db->getQuery(true);
			$query->select('*');
			$query->from('#__hikashop_address');
			$query->where('address_user_id='.$local_hikashiop_user_id);
			$query->where('address_default=1');
			$db->setQuery($query);
			$local_hikashop_address=$db->loadObject();
			
			if(is_null($local_hikashop_address)) {
				$query=$db->getQuery(true);
				$query->insert('#__hikashop_address');
				$query->set('address_user_id='.$local_hikashiop_user_id);
				$query->set('address_default=1');	
			} else {
				$query=$db->getQuery(true);
				$query->update('#__hikashop_address');
				$query->where('address_user_id='.$local_hikashiop_user_id);
				$query->where('address_default=1');
				$query->set('address_default=1');
			}
			unset($mojeid_hikashop_address->address_id);
			unset($mojeid_hikashop_address->address_user_id);
				
			foreach(get_object_vars($mojeid_hikashop_address) as $key=>$val) {
				if(!empty($val)) {
					$query->set($key.'='.$db->quote($val));
				}	
			}
			
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			if(is_null($local_hikashop_address)) {
				$local_bt_address_id=$db->insertid();
			} else {
				$local_bt_address_id=$local_hikashop_address->address_id;
			}
			
			if(is_null($moje_id_bt_address_id)) {
				$query=$db->getQuery(true);
				$query->insert('#__hika_address_mojeid');
				$query->set('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
				$query->set('address_type='.$db->quote('BT'));
				$query->set('address_id='.$local_bt_address_id);
				$db->setQuery($query);
				if(!$db->query()) {
					$this->setError($db->stderr(true));
					return falsee;
				}
			} else {
				$query=$db->getQuery(true);
				$query->update('#__hika_address_mojeid');
				$query->set('address_id='.$local_bt_address_id);
				$query->where('address_id='.$moje_id_bt_address_id);
				$db->setQuery($query);
				if(!$db->query()) {
					$this->setError($db->stderr(true));
					return false;
				}
			}
			
			
			$query=$db->getQuery(true);
			$query->select('address_id');
			$query->from('#__hika_address_mojeid');
			$query->where('mojeid_ident='.$db->quote($result->getDisplayIdentifier()));
			$query->where('address_type='.$db->quote('ST'));
			$db->setQuery($query);
			$moje_id_st_address_id=$db->loadResult();
			
			if(!is_null($moje_id_st_address_id)) {
				$query=$db->getQuery(true);
				$query->update('#__hikashop_address');
				$query->set('address_user_id='.$local_hikashiop_user_id);
				$query->where('address_user_id='.$mojeid_hikashop_user_id);
				$query->where('address_id='.$moje_id_st_address_id);
				$db->setQuery($query);
				if(!$db->query()) {
					$this->setError($db->stderr(true));
					return false;	
				}
			}
			
			$query=$db->getQuery(true);
			$query->update('#__hikashop_address');
			$query->set('address_user_id='.$local_hikashiop_user_id);
			$query->where('address_user_id='.$mojeid_hikashop_user_id);
			$query->where('address_default!=1');
			if(!is_null($local_bt_address_id)) {
				$query->where('address_id!='.$local_bt_address_id);
			}
			$db->setQuery($query);		
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);	
			$query->update('#__hikashop_cart');
			$query->set('user_id='.$user->get('id'));
			$query->where('user_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__hikashop_order');
			$query->set('order_user_id='.$local_hikashiop_user_id);
			$query->where('order_user_id='.$mojeid_hikashop_user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__hikashop_vote');
			$query->set('vote_user_id='.$local_hikashiop_user_id);
			$query->where('vote_user_id='.$mojeid_hikashop_user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__hikashop_vote_user');
			$query->set('vote_user_user_id='.$local_hikashiop_user_id);
			$query->where('vote_user_user_id='.$mojeid_hikashop_user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->delete('#__hikashop_address');
			$query->where('address_user_id='.$mojeid_hikashop_user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->delete('#__hikashop_user');
			$query->where('user_cms_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__hikashop_user');
			$query->set('user_email='.$db->quote($user->get('email')));
			$query->where('user_cms_id='.$user->get('id'));
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;	
			}
		}
		return true;
	}
	
	private function _saveSPACEShop($user,&$response,$result,$old_local_exists,$mojeid_o) {
		$name=explode(" ",$response->fullname);
		
		$fields=array();
		$fields['firstname']=$name[0];
		$fields['surname']=$name[1];
		
		$ax=Auth_OpenID_AX_FetchResponse::fromSuccessResponse($result);
		foreach($ax->data as $uri=>$vals) {
			$name=str_replace("http://specs.nic.cz/attr/","",$uri);
			switch($name) {
				case 'addr/bill/street':
					if(isset($vals[0])) {
						$fields['street']=$vals[0];
					}
					break;
				case 'addr/bill/city':
					if(isset($vals[0])) {
						$fields['city']=$vals[0];
					}
					break;
				case 'addr/bill/pc':
					if(isset($vals[0])) {
						$fields['postcode']=$vals[0];
					}
					break;
				case 'addr/bill/cc':
					if(isset($vals[0])) {
						$query=$db->getQuery(true);
						$query->select('id');
						$query->from('#__ss_countries');
						$query->where('country_2_code='.$db->quote($vals[0]));
						$db->setQuery($query);
						$fields['country_id']=$db->loadResult();
					}
					break;							
				case 'contact/ident/vat_id':
					if(isset($vals[0])) {
						$fields['ico']=$vals[0];
					}
					break;
				case 'contact/vat':
					if(isset($vals[0])) {
						$fields['dic']=$vals[0];
					}
					break;
						
			}
		}
		
		foreach($fields as $key=>$val) {
			$query=$db->getQuery(true);
			$query->select('COUNT(*)');
			$query->from('#__user_profiles');
			$query->where('user_id='.$user->get('id'));
			$query->where('profile_key='.$db->quote('spaceshop.'.$key));
			$db->setQuery($query);
			$new=!(bool)$db->loadResult();
		
			$query=$db->getQuery(true);
			if($new) {
				$query->insert('#__user_profiles');
				$query->set('user_id='.$user->get('id'));
			} else {
				$query->update('#__user_profiles');
			}
			$query->set('profile_key='.$db->quote('spaceshop.'.$key));
			$query->set('profile_value='.$db->quote(json_encode($val)));			
			
			if(!$new) {
				$query->where('user_id='.$user->get('id'));	
				$query->where('profile_key='.$db->quote('spaceshop.'.$key));
			}
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
		}
		
		if($old_local_exists) {
			$query=$db->getQuery(true);
			$query->select('REPLACE(profile_key,"spaceshop.","") AS pkey, profile_value');
			$query->from('#__user_profiles');
			$query->where('user_id='.$mojeid_o->user_id);
			$query->where('profile_key LIKE ('.$db->quote('spaceshop.%').')');
			$db->setQuery($query);
			$profile=$db->loadObjectList('pkey');
			
			foreach($profile as $key=>$val) {
				$query=$db->getQuery(true);
				$query->select('profile_value');
				$query->from('#__user_profiles');
				$query->where('user_id='.$user->get('id'));
				$query->where('profile_key='.$db->quote('spaceshop.'.$key));
				$db->setQuery($query);
				$profile_value=$db->loadResult();
				if($val->profile_value!='""') {
					$query=$db->getQuery(true);
					$query->select('COUNT(*)');
					$query->from('#__user_profiles');
					$query->where('user_id='.$user->get('id'));
					$query->where('profile_key='.$db->quote('spaceshop.'.$key));
					$db->setQuery($query);
					$cnt=$db->loadResult();
					
					$query=$db->getQuery(true);
					if($cnt) {
						$query->update('#__user_profiles');
						$query->where('user_id='.$user->get('id'));
						$query->where('profile_key='.$db->quote('spaceshop.'.$key));	
					} else {
						$query->insert('#__user_profiles');
						$query->set('user_id='.$user->get('id'));
						$query->set('profile_key='.$db->quote('spaceshop.'.$key));
					}
					$query->set('profile_value='.$db->quote($val->profile_value));
					$db->setQuery($query);
					if(!$db->query()) {
						$this->setError($this->_db->stderr(true));
						return false;
					}
				}
			}
			
			$query=$db->getQuery(true);
			$query->delete('#__user_profiles');
			$query->where('user_id='.$mojeid_o->user_id);
			$query->where('profile_key LIKE ('.$db->quote('spaceshop.%').')');
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
			
			$query=$db->getQuery(true);
			$query->update('#__ss_orders');
			$query->set('customer_id='.$user->get('id'));
			$query->where('customer_id='.$mojeid_o->user_id);
			$db->setQuery($query);
			if(!$db->query()) {
				$this->setError($db->stderr(true));
				return false;
			}
		}
		return true;
	}
}
?>